import {
  getCircleRadius,
  getContinentColor,
  gdpScale,
  lifeScale,
  tooltip,
  factBox,
  slider,
} from './setup.js';

export function updateYear(
  data,
  facts,
  curYear,
  curContinent,
) {
  slider.property('value', curYear);
  let fact = facts.get(curYear);
  if (
    fact &&
    (curContinent == 'global' ||
      fact.country == 'global' ||
      data.get(fact.country).continent == curContinent)
  ) {
    setFactBox(fact, data, curYear, curContinent);
    factBox.transition().duration(200).style('opacity', 1);
  } else {
    factBox.transition().duration(1000).style('opacity', 0);
  }
  d3.select('#yearLabel').text(`Year: ${curYear}`);
  d3.select('#sliderLabel').text(`Year: ${curYear}`);
}

export function popUpTooltip(event, data, year) {
  tooltip
    .style('opacity', 1)
    // I can use the z-index style propert to place the tooltip on top of everything else
    // when I want to render it...
    .style('z-index', 10)
    .html(
      `<strong style="font-size: 1.2em;">${data.country}</strong><br/>
      Pop (millions): ${Math.ceil(data.population.get(year) / 1000000)}<br/>
      GDP per capita: ${data.income.get(year)}<br/>
      Mean lifespan: ${data.life_exp.get(year)}`,
    );
}

export function hideTooltip() {
  // and then use z-index to put it behind everything else when I'm done (to prevent having
  // an invisible tooltip on top of my circles interfering with mouse events)
  tooltip.style('opacity', 0).style('z-index', -1);
}

export function adjustTooltip(event) {
  // Since the tooltip div has "position: absolute" styling (from style.css), setting the "left"
  // and "top" styles will place it directly at this pixel location on my screen
  tooltip
    .style('left', event.x - 160 + 'px')
    .style('top', event.y - 120 + 'px');
}

export function highlightMarks(obj, data, year) {
  let continent = d3.select(obj).attr('class');

  // Select just the country scatter point (that triggered the event)
  d3.select(obj)
    .attr('fill', 'red')
    .attr('opacity', 1.0)
    .attr('r', () => getCircleRadius(data, year) * 1.5);

  // Select *every* line
  d3.selectAll('path.line').attr('opacity', 0.0);

  // Select all continent lines
  d3.selectAll(`path.${continent}.line`).attr(
    'opacity',
    0.2,
  );

  // Select just the country line; I'm doing this after I handle all the
  // continent lines so I overwrite specific properties for just the country
  d3.select(`path#${obj.id}`)
    .attr('stroke', 'black')
    .attr('opacity', 1.0)
    .attr('stroke-width', 3);
}

export function resetMarks(obj, data, year, country) {
  let continent = d3.select(obj).attr('class');

  // Select just the country scatter point (that triggered the event)
  d3.select(obj)
    .attr('fill', (d) => getContinentColor(d, country))
    .attr('opacity', 0.7)
    .attr('r', () => getCircleRadius(data, year));

  d3.selectAll('path.line').attr('opacity', 0.2);

  // Select just the country line
  d3.selectAll(`path#${obj.id}`)
    .attr('stroke', (d) => getContinentColor(d, country))
    .attr('stroke-width', 1.5);
}

export function setFactBox(
  fact,
  data,
  curYear,
  curContinent,
) {
  factBox.html(fact.text);
  if (fact.country == 'global') {
    // For global facts, jsut draw them above the line plot
    factBox.style('left', '500px').style('top', '0px');
  } else {
    // But for facts linked to a specific country, actually calculate where that country's
    // circle is in the scatter plot, and put the annotation next to it
    let gdp = data.get(fact.country).income.get(curYear);
    let life = data.get(fact.country).life_exp.get(curYear);
    factBox
      .style('left', gdpScale(gdp) + 'px')
      .style('top', lifeScale(life) + 'px');
  }
}
