function normalizeCountry(name) {
  return name
    .replaceAll(' ', '_')
    .replaceAll(',', '')
    .replaceAll('.', '')
    .replaceAll("'", '');
}

export function processRawData(rawData) {
  let data = new Map();
  // I'm basically just going to tranpose my data: instead of each object holding data for a single
  // year (across all countries), I want my objects to hold data for a single country (across all
  // years). This will be aweseome since my circles/lines represent a single country, and I can just
  // move/adjust them without changing the bound data
  rawData.forEach((row) => {
    row.countries.forEach((country) => {
      country.country = normalizeCountry(country.country);
      if (!data.has(country.country)) {
        data.set(country.country, {
          // I want each country's data to store useful info (name of the country and continent)
          continent: country.continent,
          country: country.country,
          // And then each of these maps will use the years as keys and the corresponding quantity as values
          population: new Map(),
          life_exp: new Map(),
          income: new Map(),
        });
      }
      let countryData = data.get(country.country);
      // Finally, I want to replace all the missing (null) data with 0's to avoid errors later
      ['population', 'income', 'life_exp'].forEach(
        (attr) => {
          countryData[attr].set(
            row.year,
            // This is some cheeky short-circuiting: if the countryData has a non-null value, it will
            // be coerced to true and used as the value; otherwise (if the data is null), it will be
            // coerced to false and JavaScript will use the second value instead
            country[attr] || 0,
          );
        },
      );
    });
  });
  return data;
}
