export const width = 900;
export const height = 500;

export const width1 = width * 0.5;
export const width2 = width - width1;

export const margin = 60;

export const continents = [
  'asia',
  'europe',
  'americas',
  'africa',
];

export const slider = d3.select('#yearSlider');

export const dropdown = d3.select('#continentSelect');

dropdown
  .selectAll('[empty]')
  .data(['global', ...continents])
  .join('option')
  .attr('value', (d) => d)
  .text((d) => d);

// Let's create two svg elements and append them to my top-level div
// Here's the first svg, and I'll save the selection to the variable "svg"
export const svg1 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width1)
  .attr('height', height);

// And the second; the ".append" method returns a selection with just
// the svg in it, so I can manipulate it later (by appending or joining
// data with it)
export const svg2 = d3
  .select('#my_dataviz')
  .append('svg')
  .attr('width', width2)
  .attr('height', height);

export const tooltip = d3
  .select('#my_dataviz')
  .append('div')
  .attr('id', 'tooltip');

export const factBox = d3
  .select('#my_dataviz')
  .append('div')
  .attr('id', 'factBox');

// Cool log scale for x
export const MIN_GDP = 100;
export const MAX_GDP = 200000;
export const gdpScale = d3
  .scaleLog()
  .domain([MIN_GDP, MAX_GDP])
  .range([margin, width1 - margin]);

// Boring linear scale for y
export const MIN_LIFE = 0;
export const MAX_LIFE = 85;
export const lifeScale = d3
  .scaleLinear()
  .domain([MIN_LIFE, MAX_LIFE])
  .range([height - margin, margin]);

export const MIN_YEAR = 1800;
export const MAX_YEAR = 2014;
export const yearScale = d3
  .scaleLinear()
  .domain([MIN_YEAR, MAX_YEAR])
  .range([margin, width2 - margin]);

export const colorScale = d3
  .scaleOrdinal()
  .range(['#66c2a5', '#fc8d62', '#8da0cb', '#e78ac3']);

export const MIN_POP = 0;
export const MAX_POP = 30000000;
export const radiusScale = d3
  .scaleLinear()
  .domain([Math.sqrt(MIN_POP), Math.sqrt(MAX_POP)])
  .range([3, 10]);

export function getCircleRadius(d, year) {
  return radiusScale(Math.sqrt(d.population.get(year)));
}

export function getContinentColor(d, curCountry) {
  return d.country == curCountry
    ? 'black'
    : colorScale(d.continent);
}

export const lineGenerator = d3
  .line()
  .x((entry) => yearScale(+entry[0]))
  .y((entry) => lifeScale(entry[1]));

svg1
  .append('g')
  .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
  .call(d3.axisBottom(gdpScale).ticks(7, '~s'));

svg1
  .append('text')
  .text('GDP Per Capita')
  .attr('x', width1 / 2)
  .attr('y', height - margin / 3)
  .style('text-anchor', 'middle');

svg1
  .append('g')
  .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
  .call(d3.axisRight(lifeScale));

svg1
  .selectAll('[empty]')
  .data(continents)
  .join('rect')
  .attr('width', 25)
  .attr('height', 25)
  .attr('fill', (d) => colorScale(d))
  .attr('stroke', 'black')
  .attr('x', width1 - 100)
  .attr(
    'y',
    (d) => height - 200 + 30 * continents.indexOf(d),
  );

svg1
  .selectAll('[empty]')
  .data(continents)
  .join('text')
  .text((d) => d)
  .attr('x', width1 - 100 + 35)
  .attr(
    'y',
    (d) => height - 200 + 18 + 30 * continents.indexOf(d),
  );

svg1
  .append('text')
  .text('Average Lifespan')
  .attr('transform', 'rotate(-90)')
  .attr('x', -height / 2)
  .attr('y', margin / 2);

svg1
  .append('text')
  .text('')
  .attr('x', margin + 40)
  .attr('y', 75)
  .attr('id', 'pinnedCountry');

svg2
  .append('g')
  .attr('transform', `translate(0,${height - margin})`) // This controls the vertical position of the Axis
  .call(
    d3
      .axisBottom(yearScale)
      .ticks(5)
      .tickFormat((t) => String(t)),
  );

svg2
  .append('text')
  .text(`Year: 1800`)
  .attr('id', 'yearLabel')
  .attr('x', width1 / 2)
  .attr('y', height - margin / 3)
  .style('text-anchor', 'middle');

svg2
  .append('g')
  .attr('transform', `translate(${margin},0)`) // This controls the vertical position of the Axis
  .call(d3.axisRight(lifeScale));
